﻿using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text.Json;
using System.Text.Json.Serialization;
using InnoLabProjektDektopApp.Services;
using System.Diagnostics;
using static InnoLabProjektDektopApp.DistractionsList;
using System;
using InnoLabProjektDektopApp.Utils;


namespace InnoLabProjektDektopApp
{
    public partial class ProgramsList : Page
    {
        private Dictionary<string, List<ProcessEntry>> _data;
        private readonly string _jsonFilePath;

        public ProgramsList()
        {
            InitializeComponent();
            _jsonFilePath = System.IO.Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets", "distractingPrograms.json");
            LoadProgramList();
            RefreshRunningProcessList();
        }

        private void SearchBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            // Platzhalter ein- oder ausblenden
            if (!string.IsNullOrEmpty(SearchBox.Text))
            {
                SearchPlaceholder.Visibility = Visibility.Collapsed; // Ausblenden
            }
            else
            {
                SearchPlaceholder.Visibility = Visibility.Visible; // Einblenden
            }

            // Filterlogik für die Suchergebnisse
            string searchText = SearchBox.Text?.ToLower() ?? string.Empty;

            // Clear current UI
            ItemsPanel.Children.Clear();

            // Filter items based on search text
            if (_data != null && _data.TryGetValue("Programs", out List<ProcessEntry>? value))
            {
                var filteredItems = value.Where(
                    item => string.IsNullOrEmpty(searchText) || 
                    item.ProcessName.Contains(searchText, StringComparison.CurrentCultureIgnoreCase))
                    .ToList();

                foreach (var item in filteredItems)
                {
                    // Create a StackPanel for each filtered item
                    ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
                }
            }
        }



        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            string enteredUrl = UrlInputBox.Text.Trim();

            if (string.IsNullOrEmpty(enteredUrl))
            {
                MessageBox.Show("Please enter a URL.", "Invalid Input", MessageBoxButton.OK, MessageBoxImage.Warning);
                return;
            }

            // Add the new URL to the current category
            if (!_data.ContainsKey("Programs"))
            {
                _data["Programs"] = new List<ProcessEntry>();
            }

            // Check if the process is already in the list
            if (_data["Programs"].Any(p => p.ProcessName.Equals(enteredUrl, StringComparison.CurrentCultureIgnoreCase)))
            {
                MessageBox.Show($"Process '{enteredUrl}' is already added.", "Info", MessageBoxButton.OK, MessageBoxImage.Information);
                UrlInputBox.Clear();
                return;
            }

            _data["Programs"].Add(new ProcessEntry
            {
                ProcessName = enteredUrl,
                MainWindowTitle = enteredUrl,
                IsDistracting = true
            });

            // Save changes to the JSON
            SaveData();

            // Add to UI dynamically
            var stackPanel = new StackPanel { Orientation = Orientation.Horizontal, Margin = new Thickness(5, 2, 5, 2) };

            var checkBox = new CheckBox
            {
                Content = $"{enteredUrl}: {enteredUrl}",
                Margin = new Thickness(5),
                IsChecked = true,
                Tag = new ProcessEntry { ProcessName = enteredUrl, MainWindowTitle = enteredUrl, IsDistracting = true }
            };

            checkBox.Checked += CheckBox_CheckedChanged;
            checkBox.Unchecked += CheckBox_CheckedChanged;

            stackPanel.Children.Add(checkBox);
            ItemsPanel.Children.Add(stackPanel);

            // Clear the input field
            UrlInputBox.Clear();

            // Refresh Panel to exclude duplicates
            RefreshRunningProcessList();
        }

        private void RefreshButton_Click(object sender, RoutedEventArgs e)
        {
            RefreshRunningProcessList();
        }

        private void RefreshRunningProcessList()
        {
            var windowedProcesses = ProcessMonitor.GetWindowedProcesses();
            ItemsPanel2.Children.Clear();
            foreach (var process in windowedProcesses)
            {
                // check if the process is not already in the list
                if (_data["Programs"].Any(p => p.ProcessName.Equals(process.Name, StringComparison.CurrentCultureIgnoreCase))) continue;

                var stackPanel = new StackPanel { Orientation = Orientation.Horizontal, Margin = new Thickness(5, 2, 5, 2) };

                var button = new Button
                {
                    Content = "+",
                    Width = 30,
                    Margin = new Thickness(5)
                };

                var textBlock = new TextBlock
                {
                    Text = $"{process.MainWindowTitle} ({process.Name})",
                    Margin = new Thickness(5),
                    ToolTip = $"{process.MainWindowTitle} ({process.Name})"
                };

                button.Click += (s, args) =>
                {
                    AddProcessFromList(process.Name, process.MainWindowTitle);
                };

                stackPanel.Children.Add(button);
                stackPanel.Children.Add(textBlock);
                ItemsPanel2.Children.Add(stackPanel);
            }
        }

        private void AddProcessFromList(string processName, string mainWindowTitle)
        {
            // Check if the process is already in the list
            if (_data.TryGetValue("Programs", out List<ProcessEntry>? value) && value.Any(p => p.ProcessName.Equals(processName, StringComparison.CurrentCultureIgnoreCase)))
            {
                MessageBox.Show($"Process '{processName}' is already added.", "Info", MessageBoxButton.OK, MessageBoxImage.Information);
                return;
            }

            //show a message box to give the user the option to change the main window title
            var inputDialog = new InputDialog("Enter a new Title:", mainWindowTitle); // Assuming InputDialog is a custom dialog
            if (inputDialog.ShowDialog() == true)
            {
                mainWindowTitle = inputDialog.Input; // Get updated title
            }

            if (inputDialog.DialogResult == false) return;


            // Add the process to the list
            if (!_data.ContainsKey("Programs"))
            {
                _data["Programs"] = [];
            }
            _data["Programs"].Add(new ProcessEntry
            {
                ProcessName = processName,
                MainWindowTitle = mainWindowTitle,
                IsDistracting = true
            });

            ShowNewItemInList(processName, mainWindowTitle, true);
            RefreshRunningProcessList();

            // Save changes to the JSON
            SaveData();
        }

        private void ShowNewItemInList(string processName, string mainWindowTitle, bool isDistracting)
        {
            // Add to UI dynamically
            var newStackPanel = new StackPanel { Orientation = Orientation.Horizontal, Margin = new Thickness(5, 2, 5, 2) };
            var newCheckBox = new CheckBox
            {
                Content = $"{mainWindowTitle} ({processName})",
                Margin = new Thickness(5),
                IsChecked = isDistracting,
                ToolTip = $"{mainWindowTitle} ({processName})",
                Tag = new ProcessEntry { ProcessName = processName, MainWindowTitle = mainWindowTitle, IsDistracting = isDistracting }
            };
            newCheckBox.Checked += CheckBox_CheckedChanged;
            newCheckBox.Unchecked += CheckBox_CheckedChanged;

            var editButton = new Button
            {
                Content = "Edit",
                Margin = new Thickness(5, 0, 0, 0),
                Tag = newCheckBox // Store reference to checkbox for easier editing
            };
            editButton.Click += (s, e) => EditItemInList(processName, mainWindowTitle);

            newStackPanel.Children.Add(newCheckBox);
            newStackPanel.Children.Add(editButton);
            ItemsPanel.Children.Add(newStackPanel);
        }

        private void EditItemInList(string processName, string mainWindowTitle)
        {
            var inputDialog = new InputDialog("Enter a new Title:", mainWindowTitle);
            inputDialog.ShowDialog();
            if (inputDialog.DialogResult == false || inputDialog.Input.Length == 0) return;

            _data["Programs"].FirstOrDefault(p => p.ProcessName.Equals(processName, StringComparison.CurrentCultureIgnoreCase))!.MainWindowTitle = inputDialog.Input;

            SaveData();
            RefreshProgramList();
        }


        private void UrlInputBox_TextChanged(object sender, TextChangedEventArgs e)
        {
            // Überprüfen, ob die TextBox leer ist
            if (!string.IsNullOrEmpty(UrlInputBox.Text))
            {
                UrlPlaceholder.Visibility = Visibility.Collapsed; // Placeholder ausblenden
            }
            else
            {
                UrlPlaceholder.Visibility = Visibility.Visible; // Placeholder anzeigen
            }
        }

        public void RefreshProgramList()
        {
            ItemsPanel.Children.Clear();

            // Kategorie-Elemente laden
            var items = _data["Programs"];
            foreach (var item in items)
            {
                ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
            }

            // Überschrift setzen
            HeaderTextBlock.Text = "Programs";
        }

        public void LoadProgramList()
        {
            try
            {
                // JSON-Datei lesen
                string jsonContent = File.ReadAllText(_jsonFilePath);

                // JSON deserialisieren
                _data = JsonSerializer.Deserialize<Dictionary<string, List<ProcessEntry>>>(jsonContent);

                if (_data != null && _data.ContainsKey("Programs"))
                {
                    // Kategorie-Elemente laden
                    var items = _data["Programs"];
                    foreach (var item in items)
                    {
                        ShowNewItemInList(item.ProcessName, item.MainWindowTitle, item.IsDistracting);
                    }

                    // Überschrift setzen
                    HeaderTextBlock.Text = "Programs";
                }
                else
                {
                    MessageBox.Show($"No items found for category 'Programs'.");
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error loading items: {ex.Message}");
            }
        }


        private void CheckBox_CheckedChanged(object sender, RoutedEventArgs e)
        {
            if (sender is CheckBox checkBox && checkBox.Tag is ProcessEntry item)
            {
                // Wert in der JSON-Datenstruktur aktualisieren
                item.IsDistracting = checkBox.IsChecked ?? false;

                // Write the value into the _data dictionary
                _data["Programs"].FirstOrDefault(p => p.ProcessName.Equals(item.ProcessName, StringComparison.CurrentCultureIgnoreCase))!.IsDistracting = item.IsDistracting;

                // Änderungen in die Datei schreiben
                SaveData();
            }
        }

        private void SaveData()
        {
            try
            {
                string updatedJson = JsonSerializer.Serialize(_data, new JsonSerializerOptions { WriteIndented = true });
                File.WriteAllText(_jsonFilePath, updatedJson);
                MessageBox.Show("Data saved successfully!");
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error saving data: {ex.Message}");
            }
        }

        private void BackButton_Click(object sender, RoutedEventArgs e)
        {
            this.NavigationService.Navigate(new Distractions());
        }

        public class ProcessEntry
        {
            [JsonPropertyName("processName")]
            public required string ProcessName { get; set; }

            [JsonPropertyName("mainWindowTitle")]
            public required string MainWindowTitle { get; set; }

            [JsonPropertyName("distracting")]
            public bool IsDistracting { get; set; }
        }
    }
}