﻿using System.Diagnostics;
using System.IO;
using System.Management;
using System.Windows;
using Newtonsoft.Json;

namespace InnoLabProjektDektopApp.Services
{
    public class ProcessInfo(string processName, DateTime startTime, DateTime endTime)
    {
        public string ProcessName { get; set; } = processName;
        public DateTime StartTime { get; set; } = startTime;
        public DateTime EndTime { get; set; } = endTime;
    }

    public class BreakInfo(DateTime startTime, DateTime endTime)
    {
        public DateTime StartTime { get; set; } = startTime;
        public DateTime EndTime { get; set; } = endTime;
    }


    class ProcessMonitor
    {
        private static List<string> blockedProcesses = [];

        private readonly Dictionary<string, DateTime> processStartTimes = [];
        private readonly List<ProcessInfo> processInfoList = [];
        public bool isMonitoring = false;
        private DateTime startTime;

        private readonly List<BreakInfo> breakInfoList = [];
        public bool IsBreak { get; private set;}
        public bool IsPause { get; private set; }
        private DateTime breakStartTime;
        public int currentCycle = 1;

        public ProcessMonitor()
        {
            IsBreak = false;
            IsPause = false;
            blockedProcesses = LoadBlockedProcesses();
        }

        public async void StartMonitoring()
        {
            if (isMonitoring || IsBreak || IsPause)
            {
                return;
            }
            startTime = DateTime.Now;
            Debug.WriteLine("Started Process Monitoring");
            var query = new WqlEventQuery("__InstanceCreationEvent", new TimeSpan(0, 0, 1),
                "TargetInstance isa \"Win32_Process\"");
            var watcher = new ManagementEventWatcher { Query = query };

            isMonitoring = true;

            // check if any distracting processes are already running
            var processes = GetRunningProcessList();
            await Task.Run(() =>
            {
                foreach (var process in processes)
                {
                    if (IsProcessOnBlockedList(process.Key))
                    {
                        var result = MessageBox.Show($"Are you sure you want to get distracted by {process.Key}?", "Distracting Process Detected",
                        MessageBoxButton.YesNo,
                        MessageBoxImage.Question,
                        MessageBoxResult.No,
                        MessageBoxOptions.DefaultDesktopOnly
                        );
                        if (result == MessageBoxResult.No)
                        {
                            KillProcessesByName(process.Key);
                        }
                        else
                        {
                            TrackProcess(process.Key, DateTime.Now);
                        }
                    }
                }
            });
            
            await Task.Run(() =>
            {
                while (isMonitoring)
                {
                    // this wait is here to prevent another event from being triggered before the previous one is handled
                    var task = Task.Run(() => watcher.WaitForNextEvent());
                    if (Task.WaitAny(task, Task.Delay(100)) == 0)
                    {
                        if (!isMonitoring) break;
                        Task.Run(() => HandleNewProcess(task.Result));
                    }
                }
            });
        }

        public void StopMonitoring()
        {
            if (!isMonitoring)
            {
                return;
            }
            Debug.WriteLine("Stopping Process Monitoring");
            CalculateCurrentDistractionStage(DateTime.Now);
            isMonitoring = false;
        }

        public void StartBreak()
        {
            if (IsBreak)
            {
                return;
            }
            IsBreak = true;
            breakStartTime = DateTime.Now;
            StopMonitoring();
        }

        public void EndBreak()
        {
            if (!IsBreak)
            {
                return;
            }
            breakInfoList.Add(new BreakInfo(breakStartTime, DateTime.Now));
            IsBreak = false;
            StartMonitoring();
            currentCycle++;
        }

        public void StartPause()
        {
            if (IsPause)
            {
                return;
            }
            IsPause = true;
            breakStartTime = DateTime.Now;
            StopMonitoring();
        }

        public void EndPause()
        {
            if (!IsPause)
            {
                return;
            }
            breakInfoList.Add(new BreakInfo(breakStartTime, DateTime.Now));
            IsPause = false;
            StartMonitoring();
        }

        public void FinishSession()
        {
            StopMonitoring();
            EndBreak();

            // save the processInfoList and breakInfoList to a file
            var processInfoListFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets\\processInfoList.json");
            var processInfoListJson = JsonConvert.SerializeObject(processInfoList);
            File.WriteAllText(processInfoListFilePath, processInfoListJson);

            var breakInfoListFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets\\breakInfoList.json");
            var breakInfoListJson = JsonConvert.SerializeObject(breakInfoList);
            File.WriteAllText(breakInfoListFilePath, breakInfoListJson);

            // reset the processInfoList and breakInfoList
            processInfoList.Clear();
            breakInfoList.Clear();
            processStartTimes.Clear();
            currentCycle = 1;
        }

        public int CalculateCurrentDistractionStage(DateTime endTime)
        {
            
            TimeSpan totalDistractionTime = CalculateTotalDistractionTime();
            Debug.WriteLine("Total distraction time: {0}", totalDistractionTime);

            // Adds a dummy value to the distractedMinutes and totalMinutes so the stage starts in the middle
            double dummyTimeInMinutes = 8;
            double totalMinutes = (endTime - startTime).TotalMinutes + 2 * dummyTimeInMinutes;
            double distractedMinutes = totalDistractionTime.TotalMinutes + dummyTimeInMinutes;
            double distractionFraction = (distractedMinutes / totalMinutes);
            int stage = (int)Math.Ceiling(distractionFraction * 7);
            Debug.WriteLine($"Distraction stage: {stage}");
            return stage;
        }


        public static List<(string Name, string MainWindowTitle)> GetWindowedProcesses()
        {
            var processes = Process.GetProcesses()
                                   .Where(p => !string.IsNullOrWhiteSpace(p.MainWindowTitle))
                                   .Where(p => p.ProcessName.ToLower() != "coflow")
                                   .Select(p => (p.ProcessName, p.MainWindowTitle))
                                   .ToList();
            return processes;
        }

        private void HandleNewProcess(ManagementBaseObject e)
        {
            var processName = Path.GetFileNameWithoutExtension((string)((ManagementBaseObject)e["TargetInstance"])["Name"]);

            // Check if the process is already being tracked
            if (processStartTimes.TryGetValue(processName, out _)) return;

            processStartTimes[processName] = DateTime.Now;

            if (IsProcessOnBlockedList(processName))
            {
                var result = MessageBox.Show($"Are you sure you want to get distracted by {processName}?", "Distracting Process Detected",
                MessageBoxButton.YesNo,
                MessageBoxImage.Question,
                MessageBoxResult.No,
                MessageBoxOptions.DefaultDesktopOnly
                );

                if (result == MessageBoxResult.No)
                {
                    KillProcessesByName(processName);
                    processStartTimes.Remove(processName);
                }
                else
                {
                    TrackProcess(processName, DateTime.Now);
                }
            }
        }

        private static void KillProcessesByName(String processName)
        {
            var runningProcesses = GetRunningProcessList();
            if (!runningProcesses.TryGetValue(processName, out List<Process>? processes) || processes == null)
            {
                return; // Return early if the process name is not found or processes is null
            }

            Debug.WriteLine("Killing {0} processes with the Name {1}", processes.Count, processName);
            foreach (var process in processes)
            {
                process.Kill();
            }
        }

        private static Dictionary<string, List<Process>> GetRunningProcessList()
        {
            var processlist = Process.GetProcesses();
            var processDictionary = new Dictionary<string, List<Process>>();

            foreach (var process in processlist)
            {
                var processName = process.ProcessName;

                if (!processDictionary.TryGetValue(processName, out List<Process>? value))
                {
                    value = [];
                    processDictionary[processName] = value;
                }

                value.Add(process);
            }

            return processDictionary;
        }

        private static bool IsProcessOnBlockedList(string processName)
        {
            return blockedProcesses.Contains(processName);
        }

        private async void TrackProcess(string processName, DateTime startTime)
        {
            Debug.WriteLine($"Start monitoring the {processName} process");

            Process? process = GetRunningProcessList().TryGetValue(processName, out List<Process>? processes) ? processes[0] : null;

            if (process == null)
            {
                Debug.WriteLine("Process exited before we could track it.");
                return;
            }

            // Token to handle task cancellation
            using var cts = new CancellationTokenSource();
            var token = cts.Token;

            // Task to wait for the process to exit
            var waitForExitTask = Task.Run(() =>
            {
                process.WaitForExit();
                SaveTrackedData(processName, startTime);
            }, token);

            // Task to monitor the isMonitoring flag
            var monitorTask = Task.Run(async () =>
            {
                while (isMonitoring)
                {
                    await Task.Delay(100); // Check every 100ms
                }
            }, token);

            try
            {
                // Await the completion of either the waitForExitTask or the monitorTask
                await Task.WhenAny(waitForExitTask, monitorTask);
                cts.Cancel();
            }
            catch (OperationCanceledException)
            {
                Debug.WriteLine("Process monitoring was canceled.");
            }
        }

        private void SaveTrackedData(String processName, DateTime startTime)
        {
            processStartTimes.Remove(processName);
            var endTime = DateTime.Now;
            processInfoList.Add(new ProcessInfo(processName, startTime, endTime));
            Debug.WriteLine("Process monitoring complete.");
            Debug.WriteLine("The Process has been running for {0} seconds", endTime - startTime);
        }


        public TimeSpan CalculateTotalDistractionTime()
        {
            var tempProcessInfoList = new List<ProcessInfo>();

            // Add the earliest start time entry from processStartTimes to tempProcessInfoList
            if (processStartTimes.Count > 0)
            {
                var earliestProcess = processStartTimes.OrderBy(p => p.Value).First();
                tempProcessInfoList.Add(new ProcessInfo(earliestProcess.Key, earliestProcess.Value, DateTime.Now));
            }

            // Add all entries from processInfoList to tempProcessInfoList
            tempProcessInfoList.AddRange(processInfoList);

            var intervals = tempProcessInfoList
                .OrderBy(p => p.StartTime)
                .Select(p => (p.StartTime, p.EndTime))
                .ToList();

            if (intervals.Count == 0)
                return TimeSpan.Zero;

            var mergedIntervals = new List<(DateTime Start, DateTime End)> { intervals[0] };

            // Iterate through the intervals, starting from the second one
            foreach (var interval in intervals.Skip(1))
            {
                var last = mergedIntervals.Last();
                // If the current interval overlaps with the last merged interval, merge them
                if (interval.StartTime <= last.End)
                {
                    mergedIntervals[^1] = (last.Start, new DateTime(Math.Max(last.End.Ticks, interval.EndTime.Ticks)));
                }
                else
                {
                    // Otherwise, add the current interval as a new merged interval
                    mergedIntervals.Add(interval);
                }
            }

            // Remove overlaps with breaks
            foreach (var breakInfo in breakInfoList)
            {
                mergedIntervals = mergedIntervals.SelectMany(interval =>
                {
                    // If the interval ends before the break starts or starts after the break ends, keep the interval as is
                    if (interval.End <= breakInfo.StartTime || interval.Start >= breakInfo.EndTime)
                    {
                        return [interval];
                    }
                    // If the interval starts before the break and ends after the break, split the interval into two
                    else if (interval.Start < breakInfo.StartTime && interval.End > breakInfo.EndTime)
                    {
                        return
                        [
                            (interval.Start, breakInfo.StartTime),
                            (breakInfo.EndTime, interval.End)
                        ];
                    }
                    // If the interval starts before the break and ends during the break, adjust the end time to the start of the break
                    else if (interval.Start < breakInfo.StartTime)
                    {
                        return new List<(DateTime Start, DateTime End)> { (interval.Start, breakInfo.StartTime) };
                    }
                    // If the interval starts during the break and ends after the break, adjust the start time to the end of the break
                    else if (interval.End > breakInfo.EndTime)
                    {
                        return [(breakInfo.EndTime, interval.End)];
                    }
                    // If the interval is completely within the break, remove the interval
                    else
                    {
                        return [];
                    }
                }).ToList();
            }

            return mergedIntervals.Aggregate(TimeSpan.Zero, (sum, interval) => sum + (interval.End - interval.Start));
        }


        private static List<string> LoadBlockedProcesses()
        {
            var blockedProcessesFilePath = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "Assets\\blockedProcesses.json");
            var fileContent = File.ReadAllText(blockedProcessesFilePath);
            return JsonConvert.DeserializeObject<List<string>>(fileContent) ?? [];
        }
    }
}
