﻿using InnoLabProjektDektopApp.Services;
using System;
using Drawing = System.Drawing;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Threading;
using Forms = System.Windows.Forms;
using System.Windows.Media;
using System.Windows.Navigation;
using System.IO;
using static System.Runtime.InteropServices.JavaScript.JSType;
using System.Net.Http.Json;
using System.Text.Json;

namespace InnoLabProjektDektopApp.Utils
{
    internal class NotifyIconManager
    {
        private readonly Forms.NotifyIcon _notifyIcon = new();
        private readonly ProcessMonitor _processMonitor = App.GetProcessMonitor();
        private DispatcherTimer _countdownTimer;
        private TimeSpan _countdownTime;
        private Forms.ToolStripLabel _countdownLabel; // Reference to the countdown label
        private class Settings
        {
            public int focusPeriod { get; set; }
            public int breakPeriod { get; set; }
            public int cycles { get; set; }
            public string distractionMode { get; set; }
            public string mascotVisible { get; set; }
            public bool wordsOfAffirmation { get; set; }
            public bool insultingWords { get; set; }
        }
        private Settings _settings = new();

        public void Initialize()
        {
            _notifyIcon.Icon = new Drawing.Icon("Assets/icon.ico");
            _notifyIcon.Text = "CoFlow";
            _notifyIcon.Visible = true;
            _notifyIcon.ContextMenuStrip = new Forms.ContextMenuStrip();
            _notifyIcon.MouseClick += NotifyIcon_MouseClick;

            // Initialize the countdown timer
            _countdownTimer = new DispatcherTimer();
            _countdownTimer.Interval = TimeSpan.FromSeconds(1);
            _countdownTimer.Tick += CountdownTimer_Tick;

            RerenderContextMenu();
        }

        public void Dispose()
        {
            _notifyIcon.Dispose();
            _countdownTimer.Stop();
        }

        private void RerenderContextMenu(bool showContextMenu = false)
        {
            _notifyIcon.ContextMenuStrip.Items.Clear();
            LoadSettings();

            if (_processMonitor.isMonitoring || _processMonitor.IsBreak || _processMonitor.IsPause)
            {
                _notifyIcon.ContextMenuStrip.Items.Add(new Forms.ToolStripLabel($"Focussession {_processMonitor.currentCycle}/{_settings.cycles}", null, false));
                // Add the countdown label (initialize it if it doesn't exist)
                string label_text = (_processMonitor.IsBreak ? "Break: " : "Focus: ") + $"{_countdownTime:mm\\:ss}" + (_processMonitor.IsPause ? " (Paused)" : "");

                _countdownLabel = new Forms.ToolStripLabel(label_text);
                _notifyIcon.ContextMenuStrip.Items.Add(_countdownLabel);
                if (!_processMonitor.IsBreak && !_processMonitor.IsPause)
                {
                    _notifyIcon.ContextMenuStrip.Items.Add("Pause", Drawing.Image.FromFile("Assets/pause.png"), (sender, args) =>
                    {
                        _countdownTimer.Stop();
                        _processMonitor.StartPause();
                        RerenderContextMenu(true);
                    });
                }
                if (_processMonitor.IsPause)
                {
                    _notifyIcon.ContextMenuStrip.Items.Add("Continue", Drawing.Image.FromFile("Assets/skip.png"), (sender, args) =>
                    {
                        _countdownTimer.Start();
                        _processMonitor.EndPause();
                        RerenderContextMenu(true);
                    });
                }

                if (_processMonitor.IsBreak)
                {
                    _notifyIcon.ContextMenuStrip.Items.Add("Skip Break", Drawing.Image.FromFile("Assets/skip.png"), (sender, args) =>
                    {
                        _processMonitor.EndBreak();
                        _countdownTimer.Stop();
                        StartCountdown(TimeSpan.FromMinutes(_settings.focusPeriod));
                        RerenderContextMenu(true);
                    });
                }

                _notifyIcon.ContextMenuStrip.Items.Add("End Session", Drawing.Image.FromFile("Assets/end.png"), (sender, args) =>
                {
                    _processMonitor.StopMonitoring();
                    StopCountdown();
                    Window mainWindow = Application.Current.MainWindow;
                    NavigationService mainNavigation = ((System.Windows.Navigation.NavigationWindow)mainWindow).NavigationService;
                    mainNavigation.Navigate(new Overview());
                    RerenderContextMenu(true);
                });
            }
            else
            {
                _notifyIcon.ContextMenuStrip.Items.Add(new Forms.ToolStripLabel("Current Settings:", null, false));
                _notifyIcon.ContextMenuStrip.Items.Add(new Forms.ToolStripLabel(
                    $"Focus Duration: {_settings.focusPeriod} min | Break Duration: {_settings.breakPeriod} min | Cycles: {_settings.cycles}", null, false));
                _notifyIcon.ContextMenuStrip.Items.Add("Start Session", Drawing.Image.FromFile("Assets/start.png"), (sender, args) =>
                {
                    _processMonitor.StartMonitoring();
                    StartCountdown(TimeSpan.FromMinutes(_settings.focusPeriod));
                    RerenderContextMenu(true);
                    Window mainWindow = Application.Current.MainWindow;
                    NavigationService mainNavigation = ((System.Windows.Navigation.NavigationWindow)mainWindow).NavigationService;
                    mainNavigation.Navigate(
                        new Session(_settings.focusPeriod,
                                    _settings.breakPeriod,
                                    _settings.cycles,
                                    _settings.distractionMode,
                                    _settings.mascotVisible,
                                    _settings.wordsOfAffirmation,
                                    _settings.insultingWords));
                });
            }


            // Reopen the context menu if requested
            if (showContextMenu)
            {
                _notifyIcon.ContextMenuStrip.Show();
            }
        }

        private void NotifyIcon_MouseClick(object sender, Forms.MouseEventArgs e)
        {
            if (e.Button == Forms.MouseButtons.Left)
            {
                Window mainWindow = Application.Current.MainWindow;
                mainWindow.WindowState = WindowState.Normal;
                mainWindow.Activate();
                NavigationService mainNavigation = ((System.Windows.Navigation.NavigationWindow)mainWindow).NavigationService;
                if (!_processMonitor.isMonitoring && !_processMonitor.IsBreak)
                {
                    mainNavigation.Navigate(new Overview());
                }
            }
        }

        private void StartCountdown(TimeSpan duration)
        {
            _countdownTime = duration;
            _countdownTimer.Start();
        }

        private void StopCountdown()
        {
            _countdownTimer.Stop();
            _countdownTime = TimeSpan.Zero;
            UpdateCountdownLabel(); // Update the label to show 00:00
        }

        private void CountdownTimer_Tick(object sender, EventArgs e)
        {
            if (_countdownTime > TimeSpan.Zero)
            {
                _countdownTime = _countdownTime.Subtract(TimeSpan.FromSeconds(1));
                UpdateCountdownLabel(); // Update only the countdown label
            }
            else
            {
                if (_processMonitor.IsBreak)
                {
                    _processMonitor.EndBreak();
                    StartCountdown(TimeSpan.FromMinutes(_settings.focusPeriod));
                    RerenderContextMenu();
                }
                else
                {
                    _processMonitor.StartBreak();
                    StartCountdown(TimeSpan.FromMinutes(_settings.breakPeriod));
                    RerenderContextMenu();
                }
            }
        }

        private void UpdateCountdownLabel()
        {
            if (_countdownLabel != null)
            {
                _countdownLabel.Text = (_processMonitor.IsBreak ? "Break: " : "Focus: ") + $"{_countdownTime:mm\\:ss}" + (_processMonitor.IsPause ? " (Paused)" : "");
            }
        }

        private void LoadSettings()
        {
            string path = Path.Combine(AppDomain.CurrentDomain.BaseDirectory, "settings.json");
            if (File.Exists(path))
            {
                string jsonContent = File.ReadAllText(path);
                var settings = JsonSerializer.Deserialize<Dictionary<string, JsonElement>>(jsonContent);
                if (settings != null && settings.TryGetValue("profile1", out JsonElement profileSettings))
                {
                    _settings.focusPeriod = int.Parse(profileSettings.GetProperty("focusPeriod").GetString().Replace(" minutes", ""));
                    _settings.breakPeriod = int.Parse(profileSettings.GetProperty("breakPeriod").GetString().Replace(" minutes", ""));
                    _settings.cycles = int.Parse(profileSettings.GetProperty("cycles").GetString() ?? "1");
                    _settings.distractionMode = profileSettings.GetProperty("distractionMode").GetString() ?? "default";
                    _settings.mascotVisible = profileSettings.GetProperty("mascotVisibility").GetString() ?? "yes";
                    _settings.wordsOfAffirmation = bool.Parse(profileSettings.GetProperty("wordsOfAffirmation").GetString() ?? "true");
                    _settings.insultingWords = bool.Parse(profileSettings.GetProperty("insultingWords").GetString() ?? "false");
                }
            }
        }

    }
}